import click
from pathlib import Path
from . import tags

home = str(Path.home())


@click.group()
def cli():
    pass


@cli.command()
@click.option(
    "-c",
    "--config",
    default="{}/.heroicli.yaml".format(home),
    help="Location of heroicli.yaml config",
)
@click.option("-t", "--tag", required=True, help="Tag to search")
@click.option("-l", "--limit", default=100000, help="Limit sugestions")
@click.option("-v", "--values", is_flag=True, help="Print values")
@click.option("-j", "--json", is_flag=True, help="Implies -v, prints json results")
def tag_inspection(config, tag, limit, values, json):
    """Find all values for a specific tag"""
    tags.inspect_tag(config, tag, limit, values, json)


@cli.command()
@click.option(
    "-c",
    "--config",
    default="{}/.heroicli.yaml".format(home),
    help="Location of heroicli.yaml config",
)
@click.option("-l", "--limit", default=5000, help="Limit sugestions")
@click.option("-v", "--values", is_flag=True, help="Print values")
@click.option("-j", "--json", is_flag=True, help="Implies -v, prints json results")
def key_inspection(config, limit, values, json):
    tags.key_inspect(config, limit, values, json)


@cli.command()
@click.option(
    "-c",
    "--config",
    default="{}/.heroicli.yaml".format(home),
    help="Location of heroicli.yaml config",
)
@click.option("-l", "--limit", default=5000, help="Limit sugestions")
@click.option(
    "-t",
    "--threshold",
    default=10000,
    help="Threshold to consider a tag high cardinality",
)
def high_cardinality(config, limit, threshold):
    tags.high_cardinality(config, limit, threshold)



from . import config_util
import requests
import json

HEADERS = {"Content-type": "application/json"}


def heroic_query(query_path, query_type, query_data, config):
    url = config_util.get_heroic_url(config) + query_path
    if query_type == "post":
        r = requests.post(url, data=query_data, headers=HEADERS)
        results = json.loads(r.text)
    return results



import yaml


def load_config(config):
    with open(config) as file:
        return yaml.load(file, Loader=yaml.FullLoader)


def get_heroic_url(config):
    return load_config(config)["heroicli"]["heroic"]["url"]



import codecs
import os

from setuptools import setup

HERE = os.path.abspath(os.path.dirname(__file__))


#####
# Helper functions
#####
def read(*filenames, **kwargs):
    """
    Build an absolute path from ``*filenames``, and  return contents of
    resulting file.  Defaults to UTF-8 encoding.
    """
    encoding = kwargs.get("encoding", "utf-8")
    sep = kwargs.get("sep", "\n")
    buf = []
    for fl in filenames:
        with codecs.open(os.path.join(HERE, fl), "rb", encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


def read_requirements(filename):
    reqs_txt = read(filename)
    parsed = reqs_txt.split("\n")
    parsed = [r.split("==")[0] for r in parsed]
    return [r for r in parsed if len(r) > 0]


setup(
    name="heroicli",
    url="https://github.com/sjoeboo/heroicli",
    author="sjoeboo",
    author_email="sjoeboo@sjoeboo.com",
    maintainer="sjoeboo",
    maintainer_email="sjoeboo@sjoeboo.com",
    version="0.0.1",
    install_requires=read_requirements("requirements.txt"),
    packages=["heroicli"],
    entry_points={"console_scripts": ["heroicli=heroicli.cli:cli"]},
)



from . import heroic
import json
import click


def inspect_tag(config, tag, limit, values, j):
    """get values for a specific tag"""
    query_path = "/metadata/tag-value-suggest"
    query_type = "post"
    query_data = json.dumps({"key": tag, "limit": limit})
    query_result = heroic.heroic_query(query_path, query_type, query_data, config)
    if query_result["errors"] == []:
        if query_result["limited"] is not False:
            click.echo("Results limited, increase `limit` to see full set")
        if not values and not j:
            click.echo("Tag: {}".format(tag))
            click.echo("Values: {}".format(len(query_result["values"])))
        elif not j:
            click.echo("Tag: {}".format(tag))
            click.echo("Values:")
            for v in query_result["values"]:
                click.echo(v)
        else:
            click.echo(json.dumps(query_result["values"]))
    else:
        click.echo(query_result["errors"])


def _key_inspect(config, limit):
    query_path = "/metadata/key-suggest"
    query_type = "post"
    query_data = json.dumps({"limit": limit})
    query_result = heroic.heroic_query(query_path, query_type, query_data, config)
    keys = []
    for k in query_result["suggestions"]:
        keys.append(k["key"])
    return keys


def key_inspect(config, limit, v, j):
    keys = _key_inspect(config, limit)
    print("Found {} $keys".format(len(keys)))
    if v and not j:
        for k in keys:
            click.echo(k)
    elif j:
        click.echo(json.dumps(keys))


def high_cardinality(config, limit, threshold):
    """ find high cardinality tags """
    keys = _key_inspect(config, limit)
    print("Found {} $keys to use to break up search...".format(len(keys)))
    query_path = "/metadata/tagkey-count"
    query_type = "post"
    hc_tags = {}
    for k in keys:
        print("Searching for high cardinality tags in the {} $key".format(k))
        query_data = json.dumps({"limit": limit, "filter": ["key", k]})
        query_result = heroic.heroic_query(query_path, query_type, query_data, config)
        if query_result["errors"] != []:
            click.echo(query_result["errors"])
        if query_result["limited"] is not False:
            click.echo("Results limited, increase `limit` to see full set")
        for s in query_result["suggestions"]:
            if s["count"] >= threshold:
                hc_tags[k] = {s["key"]: s["count"]}
                # hc_tags[s['key']] = s['count']

    for k, v in hc_tags.items():
        print("In $key = {}:".format(k))
        for k2, v2 in v.items():
            print("{} - {}".format(k2, v2))
        print("")
